#!/bin/sh
# Adapted from Diomidis Spinellis's DGSH Repository: example/code-metrics.sh

# This script takes two files as input: the first is the input directory
# and the second is the output file. The script will then gather statistics
# about the code and write them to the output file.

# if there isn't exactly two arguments, print usage
if [ $# -ne 2 ]; then
    echo "Usage: $0 <input-dir> <output-file>"
    exit 1
fi

# cd into the code directory
cd $1

(

	export SGDIR=/tmp/sg-$$.0

	rm -rf $SGDIR

	# Create a temporary directory
	mkdir $SGDIR
	 {  find \( -name \*.c -or -name \*.h \) -type f -print0
}</dev/null  >$SGDIR/npi-1.0.0
ln $SGDIR/npi-1.0.0 $SGDIR/npi-1.1.0
FNAMELEN=$(  {  tr \\0 \\n |
		# Remove path
		sed 's|^.*/||' |
		# Maintain average
		awk '{s += length($1); n++} END {print s / n}' # shell is interperting $1 as argument from sed 
                                                       # but PaSh is interpreting $1 as original $1
} <$SGDIR/npi-1.0.0 )
 {  xargs -0 cat
} <$SGDIR/npi-1.1.0 >$SGDIR/npi-2.0.0
ln $SGDIR/npi-2.0.0 $SGDIR/npi-2.1.0
ln $SGDIR/npi-2.0.0 $SGDIR/npi-2.2.0
ln $SGDIR/npi-2.0.0 $SGDIR/npi-2.3.0
ln $SGDIR/npi-2.0.0 $SGDIR/npi-2.4.0
 {  sed 's/#/@/g;s/\\[\\"'\'']/@/g;s/"[^"]*"/""/g;'"s/'[^']*'/''/g" |
			cpp -P 2>/dev/null
} <$SGDIR/npi-2.0.0 >$SGDIR/npi-3.0.0
ln $SGDIR/npi-3.0.0 $SGDIR/npi-3.1.0
ln $SGDIR/npi-3.0.0 $SGDIR/npi-3.2.0
ln $SGDIR/npi-3.0.0 $SGDIR/npi-3.3.0
ln $SGDIR/npi-3.0.0 $SGDIR/npi-3.4.0

# Count the number of structs
NSTRUCT=$(  {   egrep -c 'struct[ 	]*{|struct[ 	]*[a-zA-Z_][a-zA-Z0-9_]*[       ]*{'
} <$SGDIR/npi-3.0.0 )

# Count the number of typedefs
NTYPEDEF=$(  {  grep -cw typedef
} <$SGDIR/npi-3.1.0 )

# Count the number of voids
NVOID=$(  {  grep -cw void
} <$SGDIR/npi-3.2.0 )

# Count the number of gets calls
NGETS=$(  {  grep -cw gets
} <$SGDIR/npi-3.3.0 )

# Count the number of identifiers
IDLEN=$(  {  tr -cs 'A-Za-z0-9_' '\n' |
				sort -u |
				awk '/^[A-Za-z]/ { len += length($1); n++ } END {print len / n}'
} <$SGDIR/npi-3.4.0 )

# Count the number of lines and characters
CHLINESCHAR=$(  {  wc -lc | awk '{OFS=":"; print $1, $2}'
} <$SGDIR/npi-2.1.0 )

# Count the number of characters
NCCHAR=$(  {  sed 's/#/@/g' |
			cpp -traditional -P 2>/dev/null |
			wc -c |
			awk '{OFMT = "%.0f"; print $1/1000}'
} <$SGDIR/npi-2.2.0 )


# Count the number of comments
NCOMMENT=$(  {  egrep -c '/\*|//'
} <$SGDIR/npi-2.3.0 )


# Count the number of copyright statements
NCOPYRIGHT=$(  {  grep -ci copyright
} <$SGDIR/npi-2.4.0 )
 {  find -name \*.c -type f -print0
}</dev/null  >$SGDIR/npi-4.0.0
ln $SGDIR/npi-4.0.0 $SGDIR/npi-4.1.0
 {  tr \\0 \\n
} <$SGDIR/npi-4.0.0 >$SGDIR/npi-5.0.0
ln $SGDIR/npi-5.0.0 $SGDIR/npi-5.1.0

# Count the number of files
NCFILE=$(  {  wc -l
} <$SGDIR/npi-5.0.0 )

# Count the number of directories
NCDIR=$(  {  sed 's,/[^/]*$,,;s,^.*/,,' | sort -u | wc -l
} <$SGDIR/npi-5.1.0 )
 {  xargs -0 cat
} <$SGDIR/npi-4.1.0 >$SGDIR/npi-6.0.0
ln $SGDIR/npi-6.0.0 $SGDIR/npi-6.1.0

# Count the number of lines and characters
CLINESCHAR=$(  {  wc -lc | awk '{OFS=":"; print $1, $2}'
} <$SGDIR/npi-6.0.0 )
 {  sed 's/#/@/g;s/\\[\\"'\'']/@/g;s/"[^"]*"/""/g;'"s/'[^']*'/''/g" |
			cpp -P 2>/dev/null
} <$SGDIR/npi-6.1.0 >$SGDIR/npi-7.0.0
ln $SGDIR/npi-7.0.0 $SGDIR/npi-7.1.0
ln $SGDIR/npi-7.0.0 $SGDIR/npi-7.2.0
ln $SGDIR/npi-7.0.0 $SGDIR/npi-7.3.0
ln $SGDIR/npi-7.0.0 $SGDIR/npi-7.4.0
ln $SGDIR/npi-7.0.0 $SGDIR/npi-7.5.0

# Count the number of functions
NFUNCTION=$(  {  grep -c '^{'
} <$SGDIR/npi-7.0.0 )

# Count the number of goto statements
NGOTO=$(  {  grep -cw goto
} <$SGDIR/npi-7.1.0 )

# Count the number of register variables
NREGISTER=$(  {  grep -cw register
} <$SGDIR/npi-7.2.0 )

# Count the number of macros
NMACRO=$(  {  grep -c '@[ 	]*define[ 	][ 	]*[a-zA-Z_][a-zA-Z0-9_]*('
} <$SGDIR/npi-7.3.0 )

# Count the number of include statements
NINCLUDE=$(  {  grep -c '@[ 	]*include'
} <$SGDIR/npi-7.4.0 )

# Count the number of constants
NCONST=$(  {  grep -o -h '[0-9][x0-9][0-9a-f]*' | wc -l
} <$SGDIR/npi-7.5.0 )

# Count the number of header files
NHFILE=$(  {  find -name \*.h -type f | wc -l
}</dev/null  )

# Gather the results
cat <<EOF >$2
FNAMELEN: `echo ${FNAMELEN}`
NSTRUCT: `echo ${NSTRUCT}`
NTYPEDEF: `echo ${NTYPEDEF}`
IDLEN: `echo ${IDLEN}`
CHLINESCHAR: `echo ${CHLINESCHAR}`
NCCHAR: `echo ${NCCHAR}`
NCOMMENT: `echo ${NCOMMENT}`
NCOPYRIGHT: `echo ${NCOPYRIGHT}`
NCFILE: `echo ${NCFILE}`
NCDIR: `echo ${NCDIR}`
CLINESCHAR: `echo ${CLINESCHAR}`
NFUNCTION: `echo ${NFUNCTION}`
NGOTO: `echo ${NGOTO}`
NREGISTER: `echo ${NREGISTER}`
NMACRO: `echo ${NMACRO}`
NINCLUDE: `echo ${NINCLUDE}`
NCONST: `echo ${NCONST}`
NVOID: `echo ${NVOID}`
NHFILE: `echo ${NHFILE}`
NGETS: `echo ${NGETS}`
EOF

# Clean up
rm -rf $SGDIR

)  3<&0 
