#!/bin/sh
# Adapted from Diomidis Spinellis's DGSH Repository: example/text-properties.sh

# This script takes two files as input: the first is the input file
# and the second is the output directory. The script will then compute
# the following properties of the input file:
# - the number of characters and their frequency
# - the number of words and their frequency
# - the number of digrams and their frequency
# - the number of trigrams and their frequency
# The results are written to the output directory.


# if there isn't exactly two arguments, print usage
if [ $# -ne 2 ]; then
    echo "Usage: $0 <input-file> <output-dir>"
    exit 1
fi

# clean up output directory
rm -rf $2
mkdir $2

# Consitent sorting across machines
export LC_ALL=C

# Convert input into a ranked frequency list
ranked_frequency()
{
	awk '{count[$1]++} END {for (i in count) print count[i], i}' |
	sort -rn
}

# Convert standard input to a ranked frequency list of specified n-grams
ngram()
{
	local N=$1

	perl -ne 'for ($i = 0; $i < length($_) - '$N'; $i++) {
		print substr($_, $i, '$N'), "\n";
	}' |
	ranked_frequency
}

(

	export SGDIR=/tmp/sg-$$.0

	rm -rf $SGDIR

	mkdir $SGDIR
	cat < $1  >$SGDIR/npi-0.0.0
ln $SGDIR/npi-0.0.0 $SGDIR/npi-0.1.0
ln $SGDIR/npi-0.0.0 $SGDIR/npi-0.2.0
 {  tr -cs a-zA-Z \\n
} <$SGDIR/npi-0.0.0 >$SGDIR/npi-1.0.0
ln $SGDIR/npi-1.0.0 $SGDIR/npi-1.1.0
ln $SGDIR/npi-1.0.0 $SGDIR/npi-1.2.0
 {  ngram 2 >$2/digram.txt
: ; } <$SGDIR/npi-1.0.0 >$SGDIR/npfo-none-1.0.0
 {  ngram 3 >$2/trigram.txt
: ; } <$SGDIR/npi-1.1.0 >$SGDIR/npfo-none-1.1.0
 {  ranked_frequency >$2/words.txt
: ; } <$SGDIR/npi-1.2.0 >$SGDIR/npfo-none-1.2.0
NCHARS=$(  {  wc -c
} <$SGDIR/npi-0.1.0 )
 {  sed 's/./&\
/g' |
	   # Print absolute and percentage
	   ranked_frequency |
	   awk 'BEGIN {OFMT = "%.2f"}
	   {print $1, $2, $1 / '"`echo ${NCHARS}`"' * 100}' >$2/character.txt
: ; } <$SGDIR/npi-0.2.0 >$SGDIR/npfo-none-0.2.0

# Gather the results
./sgsh-tee  -i $SGDIR/npfo-none-1.0.0  -i $SGDIR/npfo-none-1.1.0  -i $SGDIR/npfo-none-1.2.0  -i $SGDIR/npfo-none-0.2.0 >/dev/null

)  3<&0
